﻿--------------------------------------------------------------------------------------------------------
--                                          Localized global                                          --
--------------------------------------------------------------------------------------------------------
local _G = getfenv(0)

local math = _G.math
local tostring = _G.tostring
local unpack = _G.unpack
local ChatFontNormal = _G.ChatFontNormal
local CreateFrame = _G.CreateFrame
local GameTooltip = _G.GameTooltip
local GetCursorPosition = _G.GetCursorPosition
local GetFriendInfo = _G.GetFriendInfo
local GetGuildInfo = _G.GetGuildInfo
local GetMouseFocus = _G.GetMouseFocus
local GetNumFriends = _G.GetNumFriends
local GetScreenHeight = _G.GetScreenHeight
local GetScreenWidth = _G.GetScreenWidth
local InterfaceOptionsFrame_OpenToCategory = _G.InterfaceOptionsFrame_OpenToCategory
local IsShiftKeyDown = _G.IsShiftKeyDown
local UIParent = _G.UIParent
local UnitAffectingCombat = _G.UnitAffectingCombat
local UnitExists = _G.UnitExists
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost
local UnitIsFriend = _G.UnitIsFriend
local UnitIsPlayer = _G.UnitIsPlayer
local UnitIsTapped = _G.UnitIsTapped
local UnitIsTappedByPlayer = _G.UnitIsTappedByPlayer
local UnitIsUnit = _G.UnitIsUnit
local UnitName = _G.UnitName
local UnitReaction = _G.UnitReaction
local WorldFrame = _G.WorldFrame

--------------------------------------------------------------------------------------------------------
--                                           AceAddon init                                            --
--------------------------------------------------------------------------------------------------------
local MODNAME = "DogTagTooltip"
local addon = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
_G.DogTagTooltip = addon

local L	= LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local DogTag = LibStub("LibDogTag-3.0")

--------------------------------------------------------------------------------------------------------
--                                       DogTagTooltip variables                                      --
--------------------------------------------------------------------------------------------------------
local lineConfigBox
local currentOffsetX, currentOffsetY = 0, 0
local currentAnchorPoint = "TOPLEFT"
local currentAnchorType = "CURSOR"
local currentUnit
local currentNumLines = 0

local leftGameTooltipStrings, rightGameTooltipStrings = {}, {}

local anchorSmart = {
	BOTTOMLEFT = "TOPRIGHT",
	BOTTOM = "TOP",
	BOTTOMRIGHT = "TOPLEFT",
	LEFT = "RIGHT",
	RIGHT = "LEFT",
	TOPLEFT = "BOTTOMRIGHT",
	TOP = "BOTTOM",
	TOPRIGHT = "BOTTOMLEFT",
}

local bgColor = {
	guild = {0, 0.25, 0, 1},
	hostilePC = {0.25, 0, 0, 1},
	hostileNPC = {0.20, 0, 0, 1},
	neutralNPC = {0.2, 0.18, 0, 1},
	friendlyPC = {0, 0.15, 0, 1},
	friendlyNPC = {0, 0.15, 0, 1},
	other = {0, 0, 0, 1},
	dead = {0.15, 0.15, 0.15, 1},
	tapped = {0.25, 0.25, 0.25, 1},
}

local backdrop = {
	bgFile = "Interface\\Buttons\\WHITE8X8",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = false,
	edgeSize = 16,
	insets = {
		left = 4,
		right = 4,
		top = 4,
		bottom = 4,
	},
}

local profileDB
local DATABASE_DEFAULTS = {
	profile = {
		position = {
			worldAnchor = "CURSOR_TOP",
			worldOffsetX = 0,
			worldOffsetY = -30,
			worldHideInCombat = false,
			frameAnchor = "BOTTOMRIGHT",
			frameOffsetX = 0,
			frameOffsetY = 15,
			frameHideInCombat = false,
		},
		texts = {
			lefts = {
				['*'] = "",
				"[if [IsPlayer or [IsEnemy and not IsPet]] then ClassColor][NameRealm:PvPRank]", -- [1]
				L["Target:"], -- [2]
				L["Guild:"], -- [3]
				L["Guild rank:"], -- [4]
				L["Level:"], -- [5]
				L["Race:"], -- [6]
				L["Class:"], -- [7]
				L["Faction:"], -- [8]
				L["Status:"], -- [9]
				L["Health:"], -- [10]
				"[if HasMP then TypePower ':']", -- [11]
				L["Experience:"], -- [12]
				L["Time Remaining:"], -- [13]
				L["Zone:"], -- [14]
				(TALENTS or "Talents") .. ":", -- [15]
			},
			rights = {
				['*'] = "",
				"", -- [1]
				"[if Target:IsPlayer or (Target:IsEnemy and not Target:IsPet) then ClassColor(unit=Target)][if IsUnit('player', Target) then '<<" .. L["You"] .. ">>' else Target:NameRealm or '" .. L["None"] .. "']", -- [2]
				"[if Guild and Guild = 'player':Guild then Green][Guild:Angle]", -- [3]
				"[GuildRank]", -- [4]
				"[Classification] [Level]", -- [5]
				"[if IsPlayer then Race else Creature]", -- [6]
				"[if IsPlayer or [IsEnemy and not IsPet] then Class:ClassColor]", -- [7]
				"[Faction]", -- [8]
				"[Status]", -- [9]
				"[FractionalHP:Short] [PercentHP:Percent:Paren]", -- [10]
				"[if HasMP then FractionalMP:Short ' ' PercentMP:Percent:Paren]", -- [11]
				"[if XP ~= 0 then FractionalXP:Short ' ' PercentXP:Percent:Paren]", -- [12]
				"[if IsUnit('pet') then PetRemainingTime]", -- [13]
				"[Zone]", -- [14]
				"[if TalentSpec then TalentTree ' ' TalentSpec:Paren]", -- [15]
			}
		}
	}
}

local worldAnchorChoices = {
	CURSOR_BOTTOM = L["Cursor"]..L["Top"],
	CURSOR_BOTTOMRIGHT = L["Cursor"]..L["Top-left"],
	CURSOR_BOTTOMLEFT = L["Cursor"]..L["Top-right"],
	CURSOR_LEFT = L["Cursor"]..L["Right"],
	CURSOR_RIGHT = L["Cursor"]..L["Left"],
	CURSOR_TOP = L["Cursor"]..L["Bottom"],
	CURSOR_TOPRIGHT = L["Cursor"]..L["Bottom-left"],
	CURSOR_TOPLEFT = L["Cursor"]..L["Bottom-right"],
	TOPLEFT = L["Screen"]..L["Top-left"],
	TOP = L["Screen"]..L["Top"],
	TOPRIGHT = L["Screen"]..L["Top-right"],
	LEFT = L["Screen"]..L["Left"],
	RIGHT = L["Screen"]..L["Right"],
	BOTTOMLEFT = L["Screen"]..L["Bottom-left"],
	BOTTOM = L["Screen"]..L["Bottom"],
	BOTTOMRIGHT = L["Screen"]..L["Bottom-right"],
	CENTER = L["Screen"]..L["Middle"],
}

local frameAnchorChoices = {
	CURSOR_BOTTOM = L["Cursor"]..L["Top"],
	CURSOR_BOTTOMRIGHT = L["Cursor"]..L["Top-left"],
	CURSOR_BOTTOMLEFT = L["Cursor"]..L["Top-right"],
	CURSOR_LEFT = L["Cursor"]..L["Right"],
	CURSOR_RIGHT = L["Cursor"]..L["Left"],
	CURSOR_TOP = L["Cursor"]..L["Bottom"],
	CURSOR_TOPRIGHT = L["Cursor"]..L["Bottom-left"],
	CURSOR_TOPLEFT = L["Cursor"]..L["Bottom-right"],
	PARENT_BOTTOM = L["Parent frame"]..L["Top"],
	PARENT_BOTTOMRIGHT = L["Parent frame"]..L["Top-right"],
	PARENT_BOTTOMLEFT = L["Parent frame"]..L["Top-left"],
	PARENT_LEFT = L["Parent frame"]..L["Right"],
	PARENT_RIGHT = L["Parent frame"]..L["Left"],
	PARENT_TOP = L["Parent frame"]..L["Bottom"],
	PARENT_TOPRIGHT = L["Parent frame"]..L["Bottom-right"],
	PARENT_TOPLEFT = L["Parent frame"]..L["Bottom-left"],
	TOPLEFT = L["Screen"]..L["Top-left"],
	TOP = L["Screen"]..L["Top"],
	TOPRIGHT = L["Screen"]..L["Top-right"],
	LEFT = L["Screen"]..L["Left"],
	RIGHT = L["Screen"]..L["Right"],
	BOTTOMLEFT = L["Screen"]..L["Bottom-left"],
	BOTTOM = L["Screen"]..L["Bottom"],
	BOTTOMRIGHT = L["Screen"]..L["Bottom-right"],
	CENTER = L["Screen"]..L["Middle"],
}

--------------------------------------------------------------------------------------------------------
--                                        DogTagTooltip options                                       --
--------------------------------------------------------------------------------------------------------
addon.options = {
	type = "group",
	name = MODNAME,
	desc = L["Tooltip text modiffier based on DogTag."],
	args = {},
}

function addon:x_min()
	return -math.floor(GetScreenWidth()/5 + 0.5) * 5
end

function addon:x_max()
	return math.floor(GetScreenWidth()/5 + 0.5) * 5
end

function addon:y_min()
	return -math.floor(GetScreenHeight()/5 + 0.5) * 5
end

function addon:y_max()
	return math.floor(GetScreenHeight()/5 + 0.5) * 5
end

function addon:CreateOptionsPanel()
	addon.options.args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["desc"],
				},
			},
		},
		position = {
			order = 2,
			type = "group",
			name = L["Positon"],
			desc = L["Change where the tooltip is located."],
			args = {
				world = {
					order = 1,
					type = "group",
					inline = true,
					name = L["World"],
					desc = L["Options for world mouseover tooltips"],
					args = {
						anchorx = {
							order = 1,
							type = "select",
							name = L["Anchor"],
							desc = L["Choose the tooltip anchor."],
							get = function()
								return profileDB.position.worldAnchor
							end,
							set = function(key, value)
								profileDB.position.worldAnchor = value
								profileDB.position.worldOffsetX = 0
								profileDB.position.worldOffsetY = 0
							end,
							values = worldAnchorChoices,
						},
						showcombat = {
							order = 2,
							type = "toggle",
							name = L["Hide in Combat"],
							desc = L["Hide the tooltip while in combat."],
							get = function()
								return profileDB.position.worldHideInCombat
							end,
							set = function(key, value)
								profileDB.position.worldHideInCombat = value
							end,
						},
						x = {
							order = 3,
							type = "range",
							name = L["Horizontal offset"],
							desc = L["Offset of the x-axis."],
							min = addon:x_min(),
							max = addon:x_max(),
							step = 1,
							bigStep = 5,
							get = function()
								return profileDB.position.worldOffsetX
							end,
							set = function(key, value)
								profileDB.position.worldOffsetX = value
							end,
						},
						y = {
							order = 4,
							type = "range",
							name = L["Vertical offset"],
							desc = L["Offset of the y-axis."],
							min = addon:y_min(),
							max = addon:y_max(),
							step = 1,
							bigStep = 5,
							get = function(key)
								return profileDB.position.worldOffsetY
							end,
							set = function(key, value)
								profileDB.position.worldOffsetY = value
							end,
						},
					},
				},
				frame = {
					order = 2,
					type = "group",
					inline = true,
					name = L["Frame"],
					desc = L["Options for frame mouseover tooltips"],
					args = {
						anchorx = {
							order = 1,
							type = "select",
							name = L["Anchor"],
							desc = L["Choose the tooltip anchor."],
							get = function()
								return profileDB.position.frameAnchor
							end,
							set = function(key, value)
								profileDB.position.frameAnchor = value
								profileDB.position.frameOffsetX = 0
								profileDB.position.frameOffsetY = 0
							end,
							values = frameAnchorChoices,
						},
						showcombat = {
							order = 2,
							type = "toggle",
							name = L["Hide in Combat"],
							desc = L["Hide the tooltip in combat."],
							get = function()
								return profileDB.position.frameHideInCombat
							end,
							set = function(key, value)
								profileDB.position.frameHideInCombat = value
							end,
						},
						x = {
							order = 3,
							type = "range",
							name = L["Horizontal offset"],
							desc = L["Offset of the x-axis."],
							min = addon:x_min(),
							max = addon:x_max(),
							step = 1,
							bigStep = 5,
							get = function()
								return profileDB.position.frameOffsetX
							end,
							set = function(key, value)
								profileDB.position.frameOffsetX = value
							end,
						},
						y = {
							order = 4,
							type = "range",
							name = L["Vertical offset"],
							desc = L["Offset of the y-axis."],
							min = addon:y_min(),
							max = addon:y_max(),
							step = 1,
							bigStep = 5,
							get = function(key)
								return profileDB.position.frameOffsetY
							end,
							set = function(key, value)
								profileDB.position.frameOffsetY = value
							end,
						},
					},
				},
			},
		},
		texts = {
			order = 3,
			type = "group",
			name = L["DogTag texts"],
			desc = L["Change the information displayed in the tooltip."],
			args = {
				edit = {
					order = 1,
					type = "execute",
					name = L["Edit lines."],
					desc = L["Edit the line texts."],
					func = function()
						addon:CreateLineConfigBox()
					end
				},
				help = {
					order = 2,
					type = "execute",
					name = L["DogTag help"],
					desc = L["Click to pop up helpful DogTag documentation."],
					func = function()
						DogTag:OpenHelp()
					end
				},
			},
		}
	}
end

function addon:SetupOptions()
	addon.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	addon.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable(MODNAME, addon.options)

	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")
	self.optionsFrames.position = AceConfigDialog:AddToBlizOptions(MODNAME, L["Positon"], MODNAME, "position")
	self.optionsFrames.texts = AceConfigDialog:AddToBlizOptions(MODNAME, L["DogTag texts"], MODNAME, "texts")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions(MODNAME, L["Profiles"], MODNAME, "profile")

	self:RegisterChatCommand("dtt", "ShowConfig")
end

--------------------------------------------------------------------------------------------------------
--                                          DogTagTooltip core                                        --
--------------------------------------------------------------------------------------------------------
function addon:OnInitialize()
	self.db = AceDB:New("DogTagTooltipDB", DATABASE_DEFAULTS, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	profileDB = self.db.profile

	for i = 1, 30 do
		GameTooltip:AddDoubleLine(' ', ' ')
		leftGameTooltipStrings[i] = _G["GameTooltipTextLeft" .. i]
		rightGameTooltipStrings[i] = _G["GameTooltipTextRight" .. i]
	end

	GameTooltip:Show()
	GameTooltip:Hide()
end

function addon:OnEnable()
	self:CreateOptionsPanel()
	self:SetupOptions()

	self:HookScript(GameTooltip, "OnShow", "DTT_OnShow")
	self:HookScript(GameTooltip, "OnHide", "DTT_OnHide")
	self:HookScript(GameTooltip, "OnUpdate", "DTT_OnUpdate")
	self:HookScript(GameTooltip, "OnTooltipSetUnit", "DTT_OnTooltipSetUnit")
	self:SecureHook("GameTooltip_SetDefaultAnchor", "AnchorTooltip")
end

--------------------------------------------------------------------------------------------------------
--                                          DogTagTooltip Hooks                                       --
--------------------------------------------------------------------------------------------------------
-- OnShow hook
function addon:DTT_OnShow(this, ...)
	if UnitAffectingCombat("player") then
		if (profileDB.position.worldHideInCombat and this:GetOwner() ~= UIParent) or profileDB.position.frameHideInCombat then
			this:Hide()
			return
		end
	end
	addon:SetBackdrop()
	addon:SetBackgroundColor()
end

-- OnHide hook
function addon:DTT_OnHide(this, ...)
	currentUnit = nil
	this:ClearLines()
	for i = 1, currentNumLines do
		DogTag:RemoveFontString(leftGameTooltipStrings[i])
		DogTag:RemoveFontString(rightGameTooltipStrings[i])
	end
	currentNumLines = 0
end

-- SetUnit hook
function addon:DTT_OnTooltipSetUnit(this, ...)
	addon:RedoLines()
end

-- SetDefaultAnchor hook
function addon:AnchorTooltip(tooltip, parent)
	if (not tooltip or not parent) then
		return;
	end
	local anchor
	local mouseFocus = GetMouseFocus()
	if mouseFocus == WorldFrame then
		anchor = profileDB.position.worldAnchor
		self:RefreshAnchor(anchor, "world")
	else
		anchor = profileDB.position.frameAnchor
		self:RefreshAnchor(anchor, "frame")
	end

	tooltip:SetOwner(parent, "ANCHOR_NONE")
	if currentAnchorType == "PARENT" then
		tooltip:SetPoint(currentAnchorPoint, parent, anchorSmart[currentAnchorPoint], currentOffsetX, currentOffsetY)
	elseif currentAnchorType == "CURSOR" then
		self:AnchorToMouse(tooltip)
	else
		tooltip:SetPoint(currentAnchorPoint, UIParent, currentAnchorPoint, currentOffsetX, currentOffsetY)
	end
	tooltip.default = 1;
end

-- OnUpdate Hook
function addon:DTT_OnUpdate(tooltip, ...)
	-- avoid fade out
	if GameTooltip:IsOwned(UIParent) then
		local _, tooltipUnit = GameTooltip:GetUnit()
		if (not tooltipUnit and currentUnit) or (GameTooltip:GetAlpha() < 1) then
			GameTooltip:Hide()
			return
		end
	end
	-- move to cursor
	if currentAnchorType == "CURSOR" and GameTooltip:GetAnchorType() == "ANCHOR_NONE" then
		self:AnchorToMouse(tooltip)
	end
end
--------------------------------------------------------------------------------------------------------
--                                         DogTagTooltip functions                                    --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function addon:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function addon:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- Refesh anchor position variables
function addon:RefreshAnchor(anchor, frame)
	if anchor:find("^CURSOR") then
		currentAnchorType = "CURSOR"
		currentAnchorPoint = anchor:sub(8)
	elseif anchor:find("^PARENT") then
		currentAnchorType = "PARENT"
		currentAnchorPoint = anchor:sub(8)
	else
		currentAnchorType = "SCREEN"
		currentAnchorPoint = anchor
	end
	if frame == "world" then
		currentOffsetX = profileDB.position.worldOffsetX
		currentOffsetY = profileDB.position.worldOffsetY
	else
		currentOffsetX = profileDB.position.frameOffsetX
		currentOffsetY = profileDB.position.frameOffsetY
	end
end

-- Position tooltip to mouse
function addon:AnchorToMouse(tooltip)
	local x, y = GetCursorPosition()
	local scale = tooltip:GetEffectiveScale()
	tooltip:ClearAllPoints()
	x = x / scale + currentOffsetX
	y = y / scale + currentOffsetY
	tooltip:SetPoint(currentAnchorPoint, UIParent, "BOTTOMLEFT", x, y)
end

-- Redo game tooltip lines
function addon:RedoLines()
	local kwargs = { unit = "mouseover" }
	currentUnit = self:GetMouseoverUnit()
	kwargs.unit = currentUnit
	currentNumLines = GameTooltip:NumLines()

	local lefts, rights = profileDB.texts.lefts, profileDB.texts.rights
	local num = 0
	for i = 1, 30 do
		local left, right = lefts[i], rights[i]
		local left_ev, right_ev
		if left and left:trim() == '' then
			left = nil
		else
			left_ev = tostring(DogTag:Evaluate(left, "Unit", kwargs) or '')
			if left_ev:trim() == '' then
				left_ev = nil
			end
		end
		if right and right:trim() == '' then
			right = nil
		else
			right_ev = tostring(DogTag:Evaluate(right, "Unit", kwargs) or '')
			if right_ev:trim() == '' then
				right_ev = nil
			end
		end

		if right then
			if right_ev or (left and left_ev and left:find("%[")) then
				num = num + 1
				if num <= currentNumLines then
					leftGameTooltipStrings[num]:SetText(left_ev or ' ')
					rightGameTooltipStrings[num]:SetText(right_ev)
					leftGameTooltipStrings[num]:Show()
					rightGameTooltipStrings[num]:Show()
					leftGameTooltipStrings[num]:SetTextColor(1,1,1)
					rightGameTooltipStrings[num]:SetTextColor(1,1,1)
				else
					GameTooltip:AddDoubleLine(left_ev or ' ', right_ev, 1, 1, 1, 1, 1, 1)
				end
				local leftFS = leftGameTooltipStrings[num]
				local rightFS = rightGameTooltipStrings[num]
				DogTag:AddFontString(leftFS, GameTooltip, left or ' ', "Unit", kwargs)
				DogTag:AddFontString(rightFS, GameTooltip, right, "Unit", kwargs)
			end
		elseif left then
			if left_ev then
				num = num + 1
				if num <= currentNumLines then
					leftGameTooltipStrings[num]:SetText(left_ev)
					leftGameTooltipStrings[num]:Show()
					leftGameTooltipStrings[num]:SetTextColor(1,1,1)
				else
					GameTooltip:AddLine(left_ev, 1, 1, 1, true)
				end
				local leftFS = leftGameTooltipStrings[num]
				DogTag:AddFontString(leftFS, GameTooltip, left, "Unit", kwargs)
			end
		end
	end
	currentNumLines = num
end

-- Get tooltip unit
function addon:GetMouseoverUnit()
	local _, tooltipUnit = GameTooltip:GetUnit()
	if not tooltipUnit or not UnitExists(tooltipUnit) or UnitIsUnit(tooltipUnit, "mouseover") then
		return "mouseover"
	else
		return tooltipUnit
	end
end

-- Set tooltip apperance
function addon:SetBackdrop()
	local original = GameTooltip:GetBackdrop()
	if original and (original.bgFile ~= backdrop.bgFile or
					original.edgeFile ~= backdrop.edgeFile or
					original.edgeSize ~= backdrop.edgeSize or
					original.insets.left ~= backdrop.insets.left) then
		--changed
		GameTooltip:SetBackdrop(backdrop)
	end
end

-- Set background color
function addon:SetBackgroundColor()
	local col = 'other'
	if currentUnit and UnitExists(currentUnit) then
		if UnitIsDeadOrGhost(currentUnit) then
			col = 'dead'
		elseif UnitIsTapped(currentUnit) and not UnitIsTappedByPlayer(currentUnit) then
			col = 'tapped'
		elseif UnitIsPlayer(currentUnit) then
			if UnitIsFriend("player", currentUnit) then
				local playerGuild = GetGuildInfo("player")
				if playerGuild and playerGuild == GetGuildInfo(currentUnit) or UnitIsUnit("player", currentUnit) then
					col = 'guild'
				else
					local friend = false
					local name = UnitName(currentUnit)
					for i = 1, GetNumFriends() do
						if GetFriendInfo(i) == name then
							friend = true
							break
						end
					end
					if friend then
						col = 'guild'
					else
						col = 'friendlyPC'
					end
				end
			else
				col = 'hostilePC'
			end
		else
			if UnitIsFriend("player", currentUnit) then
				col = 'friendlyNPC'
			else
				local reaction = UnitReaction(currentUnit, "player")
				if not reaction or reaction <= 2 then
					col = 'hostileNPC'
				else
					col = 'neutralNPC'
				end
			end
		end
	end
	GameTooltip:SetBackdropColor(unpack(bgColor[col]))
end

-- Clean editted lines
function addon:CleanCode(text)
	text = DogTag:CleanCode(text)
	text = text:gsub("%s%s*", " ")
	return text
end

-- Text lines edit frame
function addon:CreateLineConfigBox()
	if not lineConfigBox then
		lineConfigBox = CreateFrame("Frame", "DogTagLineConfigFrame", UIParent, "DialogBoxFrame")
		lineConfigBox:SetFrameStrata("FULLSCREEN_DIALOG")
		lineConfigBox:SetFrameLevel(40)

		lineConfigBox:SetToplevel(false)
		lineConfigBox:SetWidth(750)
		lineConfigBox:SetHeight(630)
		lineConfigBox:SetPoint("CENTER")
		lineConfigBox:SetBackdrop({
			bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
		    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		    tile = true, tileSize = 16, edgeSize = 16,
		    insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
		lineConfigBox:SetBackdropColor(0,0,0,1)

		lineConfigBox:EnableMouse(true)
		lineConfigBox:SetMovable(true)
		lineConfigBox:RegisterForDrag("LeftButton")
		lineConfigBox:SetClampedToScreen(true)
		lineConfigBox:SetScript("OnDragStart", function(this)
			this:StartMoving()
		end)
		lineConfigBox:SetScript("OnDragStop", function(this)
			this:StopMovingOrSizing()
		end)

		local helpButton = CreateFrame("Button", lineConfigBox:GetName() .. "_HelpButton", lineConfigBox, "UIPanelButtonTemplate")
		helpButton:SetText(L["Help"])
		helpButton:SetScript("OnClick", function(this)
			DogTag:OpenHelp()
		end)
		helpButton:SetPoint("BOTTOMRIGHT", -12, 12)

		local GetCaretPosition, SetCaretPosition

		local hasFocus
		local lastText
		local function OnEditFocusGained(this)
			hasFocus = this
			lastText = this:GetText()
		end

		local function OnEditFocusLost(this)
			if hasFocus == this then
				hasFocus = nil
				lastText = nil
			end
			local num, side = this.num, this.side
			local text = this:GetText() or ''

			text = self:CleanCode(text)
			this:SetText(DogTag:ColorizeCode(text))

			profileDB.texts[side == 'left' and 'lefts' or 'rights'][num] = text
		end

		local pipe_byte = ("|"):byte()
		local c_byte = ("c"):byte()
		local r_byte = ("r"):byte()

		local function OnTextChanged(this)
			if hasFocus == this then
				local text = this:GetText()
			 	if lastText ~= text then
					lastText = text
					local position = GetCaretPosition(this)
					local skip = 0
					for i = 1, position do
						if text:byte(i) == pipe_byte then
							if text:byte(i+1) == c_byte then
								skip = skip + 10
							elseif text:byte(i+1) == r_byte then
								skip = skip + 2
							end
						end
					end
					position = position - skip
					lastText = DogTag:ColorizeCode(lastText)
					this:SetText(lastText)
					local betterPosition = 0
					for i = 1, position do
						betterPosition = betterPosition + 1
						while lastText:byte(betterPosition) == ("|"):byte() do
							if lastText:byte(betterPosition+1) == ("c"):byte() then
								betterPosition = betterPosition + 10
							elseif lastText:byte(betterPosition+1) == ("r"):byte() then
								betterPosition = betterPosition + 2
							else
								break
							end
						end
					end

					SetCaretPosition(this, betterPosition)
				end
			end
		end

		function GetCaretPosition(this)
			local text = this:GetText()
			if #text == 0 then
				return 0
			end
			this:SetScript("OnTextChanged", nil)
			this:Insert("\001")
			local position = this:GetText():find("\001", 1, 1)
			this:SetText(text)

			if position then
				this:SetText(text:sub(1, position - 1) .. "X" .. text:sub(position))
				this:HighlightText(position - 1, position)
				this:Insert("\000")
			end
			this:SetScript("OnTextChanged", OnTextChanged)

			return (position or 0) - 1
		end

		function SetCaretPosition(this, position)
			local text = this:GetText()
			if #text == 0 then
				return
			end
			this:SetScript("OnTextChanged", nil)
			this:SetText(text:sub(1, position) .. "X" .. text:sub(position + 1))
			this:HighlightText(position, position + 1)
			this:Insert("\000")
			this:SetScript("OnTextChanged", OnTextChanged)
		end

		local function OnEnterPressed(this)
			this:ClearFocus()
		end

		local function OnTabPressed(this)
			local num, side = this.num, this.side

			this:ClearFocus()

			if not IsShiftKeyDown() then
				if side == "left" then
					this:GetParent()["right" .. num]:SetFocus()
				elseif num < 30 then
					this:GetParent()["left" .. (num+1)]:SetFocus()
				end
			else
				if side == "right" then
					this:GetParent()["left" .. num]:SetFocus()
				elseif num > 1 then
					this:GetParent()["right" .. (num-1)]:SetFocus()
				end
			end
		end

		for i = 1, 30 do
			for j = 1, 2 do
				local leftSide = j == 1
				local editBox = CreateFrame("EditBox", "DogTagLineConfigFrameEditBox" .. (leftSide and "Left" or "Right") .. i, lineConfigBox)
				local last
				if leftSide then
					lineConfigBox["left" .. i] = editBox
					last = lineConfigBox["left" .. i-1]
				else
					lineConfigBox["right" .. i] = editBox
					last = lineConfigBox["right" .. i-1]
				end
				editBox:SetFontObject(ChatFontNormal)
				editBox:SetHeight(13)
				if last then
					editBox:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -5)
					editBox:SetPoint("TOPRIGHT", last, "BOTTOMRIGHT", 0, -5)
				else
					if leftSide then
						editBox:SetPoint("TOPLEFT", lineConfigBox, "TOPLEFT", 16, -20)
						editBox:SetPoint("TOPRIGHT", lineConfigBox, "TOP", -8, -20)
					else
						editBox:SetPoint("TOPLEFT", lineConfigBox, "TOP", 8, -20)
						editBox:SetPoint("TOPRIGHT", lineConfigBox, "TOPRIGHT", -16, -20)
					end
				end
				editBox:SetJustifyH("LEFT")

				local width = editBox:GetWidth()/2 + 10
				local underline = editBox:CreateTexture(nil, "BACKGROUND")
				editBox.underline = underline
				underline:SetTexture("Interface\\Buttons\\WHITE8X8")
				underline:SetHeight(1)
				underline:SetPoint("TOPLEFT", editBox, "BOTTOMLEFT", 0, -1)
				underline:SetPoint("TOPRIGHT", editBox, "BOTTOMRIGHT", 0, -1)
				editBox:SetAutoFocus(false)

				editBox:SetScript("OnEscapePressed", editBox.ClearFocus)
				editBox.num = i
				editBox.side = leftSide and "left" or "right"

				editBox:SetScript("OnEnterPressed", OnEnterPressed)
				editBox:SetScript("OnEditFocusLost", OnEditFocusLost)
				editBox:SetScript("OnEditFocusGained", OnEditFocusGained)
				editBox:SetScript("OnTextChanged", OnTextChanged)

				editBox:SetScript("OnTabPressed", OnTabPressed)
			end
		end
	end
	lineConfigBox:Show()
	for i = 1, 30 do
		local left = profileDB.texts.lefts[i]
		local right = profileDB.texts.rights[i]
		lineConfigBox["left" .. i]:SetText(DogTag:ColorizeCode(self:CleanCode(left)))
		lineConfigBox["right" .. i]:SetText(DogTag:ColorizeCode(self:CleanCode(right)))
	end
end
